/**
 * @file Vector2.h
 * @author Gawen ACKERMANN and Florian BURGENER
 * @brief Set of functions and structures allowing the manipulation of a vector 2.
 * @version 1.0
 * @date 2021-12-19
 *
 */

#ifndef VECTOR2_H
#define VECTOR2_H

#include <stdbool.h>
#include <stdint.h>

typedef struct Vector2 {
    double x;
    double y;
} Vector2;

/**
 * @brief Creates a new Vector2.
 *
 * @param x
 * @param y
 * @return Vector2
 */
Vector2 vector2_create(double x, double y);

/**
 * @brief Creates a new Vector2 where x=0 and y=0.
 *
 * @return Vector2
 */
Vector2 vector2_create_zero();

/**
 * @brief Sum two vectors and return the result in a new one.
 *
 * @param a
 * @param b
 * @return Vector2
 */
Vector2 vector2_add(Vector2 a, Vector2 b);

/**
 * @brief Subtract two vectors and return the result in a new one.
 *
 * @param a
 * @param b
 * @return Vector2
 */
Vector2 vector2_substract(Vector2 a, Vector2 b);

/**
 * @brief Multiply two vectors and return the result in a new one.
 *
 * @param v
 * @param scalar
 * @return Vector2
 */
Vector2 vector2_multiply(Vector2 v, double scalar);

/**
 * @brief Computes the dot product of two vectors.
 *
 * @param a
 * @param b
 * @return double
 */
double vector2_dot_product(Vector2 a, Vector2 b);

/**
 * @brief Calculates the norm of a vector and returns the square of the norm.
 *
 * @param v
 * @return double
 */
double vector2_norm_sqr(Vector2 v);

/**
 * @brief Calculates the norm of a vector.
 *
 * @param v
 * @return double
 */
double vector2_norm(Vector2 v);

/**
 * @brief Normalizes a vector and return the result in a new one.
 *
 * @param v
 * @return Vector2
 */
Vector2 vector2_normalize(Vector2 v);

/**
 * @brief Convert a vector (-1 to 1) into a width x height rectangle.
 *
 * @param v
 * @param width
 * @param height
 * @return Vector2
 */
Vector2 vector2_fit_canvas(Vector2 v, int32_t width, int32_t height);

/**
 * @brief Displays a vector in the console.
 *
 * @param v
 */
void vector2_print(Vector2 v);

#endif
